import { Component, forwardRef, Attribute, ViewChild, Pipe, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

class MarkdownEditorComponent {
    constructor(required = false, maxlength = -1) {
        this.required = required;
        this.maxlength = maxlength;
        /** flag for fullscreen mode */
        this.fullscreenEnabled = false;
        /** flag for show/hide preview area */
        this.previewEnabled = true;
        /** flag for disabling the editor (and all its controls) */
        this.disabled = false;
        this.onChange = (newValue) => { };
        this.onTouched = () => { };
    }
    get value() {
        return this._value || '';
    }
    set value(value) {
        this._value = value;
        this.parsedHtml = marked(this._value);
        this.onChange(value);
    }
    ngOnInit() {
    }
    ngAfterViewInit() {
        this.codeMirror = CodeMirror.fromTextArea(this.editor.nativeElement, {
            mode: 'markdown',
            lineNumbers: true,
            autoCloseTags: true,
            lineWrapping: true,
            extraKeys: { 'Ctrl-Space': 'autocomplete' }
        });
        this.codeMirror.doc.on('change', (instance, change) => {
            this.value = instance.getValue();
        });
        this.resizeCodeMirror();
    }
    /* ControlValueAccessor methods  */
    writeValue(obj) {
        this._value = '';
        if (typeof obj === 'string' && this.codeMirror) {
            this.codeMirror.setValue(obj);
            this.onChange(obj);
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        if (this.codeMirror) {
            this.codeMirror.setOption('readOnly', isDisabled);
        }
    }
    /* END ControlValueAccessor methods  */
    /**
     * Validate implementation
     *
     * only required and maxlength for now
     */
    validate(control) {
        let result = null;
        if (this.required && this._value.length === 0) {
            result = { required: true };
        }
        if (this.maxlength > 0 && this.value.length > this.maxlength) {
            result = { maxlength: true };
        }
        return result;
    }
    /**
     * Resizes the code mirror instance to the size of the container
     *
     * Used when the code mirror editor element size is changed (window resize, )
     */
    resizeCodeMirror() {
        setTimeout(() => {
            const editorContainer = this.editor.nativeElement.parentNode;
            this.codeMirror.setSize(editorContainer.offsetWidth, editorContainer.offsetHeight);
        }, 200);
    }
    toggleFullscreen() {
        this.fullscreenEnabled = !this.fullscreenEnabled;
        this.resizeCodeMirror();
    }
    togglePreview() {
        this.previewEnabled = !this.previewEnabled;
        this.resizeCodeMirror();
    }
    /**
     * Inserts special content into the editor
     *
     * can be used for different buttons like bold, italic,
     *
     * Override the method if you want to customize the behaviour of the
     * insertion. For example if you have an own image selector and so on.
     *
     * @param type type of the content that is to be inserted
     * @param value optional value (for special custom content)
     */
    insertContent(type, value) {
        let from = this.codeMirror.getCursor('from');
        // if value is set, use it, if not get the selection instead
        var selection = value ? value : this.codeMirror.getSelection();
        let to = this.codeMirror.getCursor('to');
        let newCursorPosition = { line: to.line, ch: to.ch };
        let additionalOffset = 0;
        var replacement = '';
        switch (type) {
            case 'bold':
                replacement = `**${selection}**`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 2;
                break;
            case 'italic':
                replacement = `*${selection}*`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 1;
                break;
            case 'quote':
                replacement = `> ${this.codeMirror.getLine(from.line)}`;
                this.codeMirror.replaceRange(replacement, { line: from.line, ch: 0 }, { line: from.line, ch: replacement.length });
                break;
            case 'list':
                replacement = `- ${this.codeMirror.getLine(from.line)}`;
                this.codeMirror.replaceRange(replacement, { line: from.line, ch: 0 }, { line: from.line, ch: replacement.length });
                break;
            case 'link':
                replacement = `[](${selection})`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 1;
                break;
            case 'image':
                replacement = `![](${selection})`;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 2;
                break;
            case 'code':
                replacement = `\`\`\`\r\n${selection}\r\n\`\`\``;
                this.codeMirror.replaceRange(replacement, from, to);
                additionalOffset = 4;
                break;
            default:
        }
        newCursorPosition.ch += replacement.length - selection.length - additionalOffset;
        this.codeMirror.setCursor(newCursorPosition);
        this.codeMirror.focus();
        this.onChange(this._value);
    }
}
MarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'maddob-markdown-editor',
                template: "<div class=\"maddob-markdown-editor-container\" [ngClass]=\"{'fullscreen': fullscreenEnabled}\">\n  \n    <!-- HEADER -->\n    <div class=\"maddob-markdown-editor-header\">\n  \n      <div class=\"maddob-markdown-editor-left\">\n        <button [disabled]=\"disabled\" (click)=\"insertContent('bold')\">bold</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('italic')\">italic</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('quote')\">quote</button>\n        <button [disabled]=\"disabled\" (click)=\"insertContent('code')\">code</button>\n      </div>\n  \n      <div class=\"maddob-markdown-editor-center\">\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('list')\">list</button>\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('link')\">link</button>\n          <button [disabled]=\"disabled\"  (click)=\"insertContent('image')\">image</button>\n        </div>\n  \n      <div class=\"maddob-markdown-editor-right\">\n        <button (click)=\"togglePreview()\">preview on</button>\n        <button (click)=\"toggleFullscreen()\">fullscreen</button>\n      </div>\n      \n    </div>\n  \n    <!-- CONTENT -->\n    <div class=\"maddob-markdown-editor-content\">\n  \n      <div class=\"maddob-markdown-editor\" (window:resize)=\"resizeCodeMirror()\"\n        [ngClass]=\"{'default-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n          <textarea #editor></textarea>\n      </div>\n  \n      <div class=\"maddob-markdown-preview\" [innerHTML]=\"parsedHtml\" *ngIf=\"previewEnabled\"\n        [ngClass]=\"{'default-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n      </div>\n  \n    </div>\n  </div>\n  ",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        multi: true,
                        useExisting: forwardRef(() => MarkdownEditorComponent),
                    },
                    {
                        provide: NG_VALIDATORS,
                        multi: true,
                        useExisting: forwardRef(() => MarkdownEditorComponent)
                    }
                ],
                styles: [".maddob-markdown-editor-container{border:1px solid #000;display:flex;flex-direction:column}.maddob-markdown-editor-header{display:flex;flex-direction:row;justify-content:space-between;padding:5px}.maddob-markdown-editor-header-left{display:flex;justify-content:flex-start}.maddob-markdown-editor-header-center{display:flex;flex-grow:0;justify-content:center}.maddob-markdown-editor-header-right{display:flex;justify-content:flex-end}.maddob-markdown-editor-content{border-top:1px solid #000;display:flex;flex-direction:row;flex-grow:1}.maddob-markdown-editor{flex-basis:50%;flex-grow:1;overflow:scroll;text-align:left}.maddob-markdown-editor textarea{height:100%;width:100%}.maddob-markdown-preview{border-left:1px solid #ccc;flex-basis:50%;overflow:scroll;padding:5px;text-align:left}.default-height{height:300px}.full-height{height:100%}.fullscreen{background-color:#fff;height:100%;left:0;overflow:hidden;position:fixed;top:0;width:100%;z-index:1000}"]
            },] }
];
MarkdownEditorComponent.ctorParameters = () => [
    { type: Boolean, decorators: [{ type: Attribute, args: ['required',] }] },
    { type: Number, decorators: [{ type: Attribute, args: ['maxlength',] }] }
];
MarkdownEditorComponent.propDecorators = {
    editor: [{ type: ViewChild, args: ["editor", { static: true },] }]
};

class UikitMarkdownEditorComponent extends MarkdownEditorComponent {
    ngOnInit() {
    }
    resizeCodeMirror() {
        setTimeout(() => {
            const editorContainer = this.editor.nativeElement.parentNode;
            this.codeMirror.setSize(editorContainer.offsetWidth - 50, editorContainer.offsetHeight);
        }, 200);
    }
}
UikitMarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'maddob-uikit-markdown-editor',
                template: "<div [ngClass]=\"{'fullscreen': fullscreenEnabled}\" class=\"full-border\">\n  <div class=\"CodeMirror-Editor-Tab uk-background-muted uk-flex\">\n    <div class=\"uk-flex uk-flex-left\">\n      <a uk-icon=\"icon: bold\" (click)=\"insertContent('bold')\" uk-tooltip=\"title: Bold; delay: 800\"></a>\n      <a uk-icon=\"icon: italic\" (click)=\"insertContent('italic')\" uk-tooltip=\"title: Italic; delay: 800\"></a>\n      <a uk-icon=\"icon: quote-right\" (click)=\"insertContent('quote')\" uk-tooltip=\"title: Blockquote; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a uk-icon=\"icon: list\" (click)=\"insertContent('list')\" uk-tooltip=\"title: List; delay: 800\"></a>\n      <a uk-icon=\"icon: code\" (click)=\"insertContent('code')\" uk-tooltip=\"title: Code; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a uk-icon=\"icon: link\" (click)=\"insertContent('link')\" uk-tooltip=\"title: Link; delay: 800\"></a>\n      <a uk-icon=\"icon: image\" (click)=\"insertContent('image')\" uk-tooltip=\"title: image; delay: 800\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left separator\"></div>\n\n    <div class=\"uk-flex uk-flex-left uk-margin-left\">\n      <a *ngIf=\"previewEnabled\" uk-icon=\"icon: chevron-right\" (click)=\"previewEnabled = false; resizeCodeMirror()\"></a>\n      <a *ngIf=\"!previewEnabled\" uk-icon=\"icon: chevron-left\" (click)=\"previewEnabled = true; resizeCodeMirror()\"></a>\n    </div>\n\n    <div class=\"uk-flex uk-flex-right\" style=\"flex-grow: 1;\">\n      <a *ngIf=\"!fullscreenEnabled\" uk-icon=\"icon: expand\" (click)=\"toggleFullscreen()\"></a>\n      <a *ngIf=\"fullscreenEnabled\" uk-icon=\"icon: shrink\" (click)=\"toggleFullscreen()\"></a>\n    </div>\n  </div>\n\n\n  <div class=\"editor-content\" class=\"uk-grid\" uk-grid\n    [ngClass]=\"{'max-height': !fullscreenEnabled, 'full-height': fullscreenEnabled}\">\n    <div class=\"code-mirror-container\"\n      [ngClass]=\"{'uk-width-1-2': previewEnabled, 'uk-width-expand': !previewEnabled, 'max-height': !fullscreenEnabled}\"\n      #codeMirrorContainer (window:resize)=\"resizeCodeMirror()\">\n      <textarea #editor name=\"content\"></textarea>\n    </div>\n    <div *ngIf=\"previewEnabled\" class=\"preview\" [ngClass]=\"{'uk-width-1-2': previewEnabled, 'hidden': !previewEnabled }\"\n      [innerHTML]=\"parsedHtml\"></div>\n  </div>\n</div>",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        multi: true,
                        useExisting: forwardRef(() => UikitMarkdownEditorComponent),
                    },
                    {
                        provide: NG_VALIDATORS,
                        multi: true,
                        useExisting: forwardRef(() => UikitMarkdownEditorComponent)
                    }
                ],
                styles: [".full-border,.html-editor-container{border:1px solid #e5e5e5}.CodeMirror-Editor-Tab{border-bottom:1px solid #e5e5e5;padding:10px}.editor-content{border:1px solid #e5e5e5}.fullscreen{height:100%;left:0;overflow:hidden;position:fixed;top:0;width:100%;z-index:1000}.preview{border-left:1px solid #e5e5e5;height:300px;overflow:scroll;overflow-wrap:break-word;padding:5px}.hidden{display:none}.max-height{height:300px;overflow:scroll}.full-height,.full-height>.uk-width-1-2>.CodeMirror,.full-height textarea{height:100%}.full-height div{background-color:#fff;display:block;height:100%}#preview-content{background:#fff;padding:5px}.uk-icon{padding:0 5px}.separator{background-color:#e5e5e5;width:1px}"]
            },] }
];

const SECONDS_MINUTE = 60;
const SECONDS_HOUR = 60 * SECONDS_MINUTE;
const SECONDS_DAY = 24 * SECONDS_HOUR;
/**
 * Transforms values in seconds in readable format
 *
 * Days is the max displayed period. If more than 2
 * period types are available, just two are displayed
 *
 * For example if hours, minutes and seconds are available,
 * seconds are omitted
 *
 * Check test cases for more details
 *
 */
class ReadableTimePipe {
    transform(value) {
        if (value < 0) {
            return '';
        }
        let seconds = value;
        let minutes = seconds / SECONDS_MINUTE;
        let hours = seconds / SECONDS_HOUR;
        let days = value / SECONDS_DAY;
        if (days > 1) {
            days = Math.floor(days);
            seconds = seconds - days * SECONDS_DAY;
            hours = seconds / SECONDS_HOUR;
        }
        if (hours > 1) {
            hours = Math.floor(hours);
            seconds = seconds - hours * SECONDS_HOUR;
            minutes = seconds / SECONDS_MINUTE;
        }
        if (minutes > 1) {
            minutes = Math.floor(minutes);
            seconds = seconds - minutes * SECONDS_MINUTE;
        }
        seconds = Math.floor(seconds);
        let parts = [];
        if (days > 1) {
            parts.push(`${days}d`);
        }
        if (hours > 1) {
            parts.push(`${hours}h`);
        }
        if (minutes > 1 && parts.length < 2) {
            parts.push(`${minutes}m`);
        }
        if (parts.length < 2) {
            parts.push(`${seconds}s`);
        }
        return parts.join(' ');
    }
}
ReadableTimePipe.decorators = [
    { type: Pipe, args: [{
                name: 'readableTime'
            },] }
];

class MaddobLibModule {
}
MaddobLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MarkdownEditorComponent, UikitMarkdownEditorComponent, ReadableTimePipe],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [MarkdownEditorComponent, UikitMarkdownEditorComponent, ReadableTimePipe]
            },] }
];

/*
 * Public API Surface of maddob-lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MaddobLibModule, MarkdownEditorComponent, ReadableTimePipe, UikitMarkdownEditorComponent };
//# sourceMappingURL=maddob-lib.js.map
